# -*- coding: utf-8 -*-
# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.


import pytest

from cryptography.x509 import (
    Name,
    NameAttribute,
    NameOID,
    RelativeDistinguishedName,
)


class TestRFC4514:
    def test_invalid(self, subtests):
        for value in [
            "C=US,CN=Joe , Smith,DC=example",
            ",C=US,CN=Joe , Smith,DC=example",
            "C=US,UNKNOWN=Joe , Smith,DC=example",
            "C=US,CN,DC=example",
            "C=US,FOOBAR=example",
        ]:
            with subtests.test():
                with pytest.raises(ValueError):
                    Name.from_rfc4514_string(value)

    def test_valid(self, subtests):
        for value, expected in [
            (
                r"CN=James \"Jim\" Smith\, III",
                Name(
                    [
                        NameAttribute(
                            NameOID.COMMON_NAME, 'James "Jim" Smith, III'
                        )
                    ]
                ),
            ),
            (
                r"UID=\# escape\+\,\;\00this\ ",
                Name([NameAttribute(NameOID.USER_ID, "# escape+,;\0this ")]),
            ),
            (
                r"2.5.4.3=James \"Jim\" Smith\, III",
                Name(
                    [
                        NameAttribute(
                            NameOID.COMMON_NAME, 'James "Jim" Smith, III'
                        )
                    ]
                ),
            ),
            ("ST=", Name([NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, "")])),
            (
                "OU=Sales+CN=J.  Smith,DC=example,DC=net",
                Name(
                    [
                        RelativeDistinguishedName(
                            [
                                NameAttribute(
                                    NameOID.ORGANIZATIONAL_UNIT_NAME, "Sales"
                                ),
                                NameAttribute(
                                    NameOID.COMMON_NAME, "J.  Smith"
                                ),
                            ]
                        ),
                        RelativeDistinguishedName(
                            [
                                NameAttribute(
                                    NameOID.DOMAIN_COMPONENT, "example"
                                )
                            ]
                        ),
                        RelativeDistinguishedName(
                            [NameAttribute(NameOID.DOMAIN_COMPONENT, "net")]
                        ),
                    ]
                ),
            ),
            (
                "CN=cryptography.io,O=PyCA,L=,ST=,C=US",
                Name(
                    [
                        NameAttribute(NameOID.COMMON_NAME, "cryptography.io"),
                        NameAttribute(NameOID.ORGANIZATION_NAME, "PyCA"),
                        NameAttribute(NameOID.LOCALITY_NAME, ""),
                        NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, ""),
                        NameAttribute(NameOID.COUNTRY_NAME, "US"),
                    ]
                ),
            ),
            (
                r"C=US,CN=Joe \, Smith,DC=example",
                Name(
                    [
                        NameAttribute(NameOID.COUNTRY_NAME, "US"),
                        NameAttribute(NameOID.COMMON_NAME, "Joe , Smith"),
                        NameAttribute(NameOID.DOMAIN_COMPONENT, "example"),
                    ]
                ),
            ),
            (
                r"C=US,CN=Jane \"J\,S\" Smith,DC=example",
                Name(
                    [
                        NameAttribute(NameOID.COUNTRY_NAME, "US"),
                        NameAttribute(NameOID.COMMON_NAME, 'Jane "J,S" Smith'),
                        NameAttribute(NameOID.DOMAIN_COMPONENT, "example"),
                    ]
                ),
            ),
            (
                'C=US,CN=\\"Jane J\\,S Smith\\",DC=example',
                Name(
                    [
                        NameAttribute(NameOID.COUNTRY_NAME, "US"),
                        NameAttribute(NameOID.COMMON_NAME, '"Jane J,S Smith"'),
                        NameAttribute(NameOID.DOMAIN_COMPONENT, "example"),
                    ]
                ),
            ),
            (
                'C=US,CN=\\"Jane \\"J\\,S\\" Smith\\",DC=example',
                Name(
                    [
                        NameAttribute(NameOID.COUNTRY_NAME, "US"),
                        NameAttribute(
                            NameOID.COMMON_NAME, '"Jane "J,S" Smith"'
                        ),
                        NameAttribute(NameOID.DOMAIN_COMPONENT, "example"),
                    ]
                ),
            ),
            (
                r"C=US,CN=Jane=Smith,DC=example",
                Name(
                    [
                        NameAttribute(NameOID.COUNTRY_NAME, "US"),
                        NameAttribute(NameOID.COMMON_NAME, "Jane=Smith"),
                        NameAttribute(NameOID.DOMAIN_COMPONENT, "example"),
                    ]
                ),
            ),
            (r"CN=#616263", Name([NameAttribute(NameOID.COMMON_NAME, "abc")])),
            (r"CN=👍", Name([NameAttribute(NameOID.COMMON_NAME, "👍")])),
            (
                "CN=\\\\123",
                Name([NameAttribute(NameOID.COMMON_NAME, "\\123")]),
            ),
            ("CN=\\\\\\;", Name([NameAttribute(NameOID.COMMON_NAME, "\\;")])),
            (
                "CN=\\\\#123",
                Name([NameAttribute(NameOID.COMMON_NAME, "\\#123")]),
            ),
            (
                "2.5.4.10=abc",
                Name([NameAttribute(NameOID.ORGANIZATION_NAME, "abc")]),
            ),
        ]:
            with subtests.test():
                result = Name.from_rfc4514_string(value)
                assert result == expected



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.


import binascii

import pytest

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.hkdf import HKDF

from .utils import wycheproof_tests


_HASH_ALGORITHMS = {
    "HKDF-SHA-1": hashes.SHA1(),
    "HKDF-SHA-256": hashes.SHA256(),
    "HKDF-SHA-384": hashes.SHA384(),
    "HKDF-SHA-512": hashes.SHA512(),
}


@wycheproof_tests(
    "hkdf_sha1_test.json",
    "hkdf_sha256_test.json",
    "hkdf_sha384_test.json",
    "hkdf_sha512_test.json",
)
def test_hkdf(backend, wycheproof):
    hash_algo = _HASH_ALGORITHMS[wycheproof.testfiledata["algorithm"]]
    if wycheproof.invalid:
        with pytest.raises(ValueError):
            HKDF(
                algorithm=hash_algo,
                length=wycheproof.testcase["size"],
                salt=binascii.unhexlify(wycheproof.testcase["salt"]),
                info=binascii.unhexlify(wycheproof.testcase["info"]),
                backend=backend,
            )
        return

    h = HKDF(
        algorithm=hash_algo,
        length=wycheproof.testcase["size"],
        salt=binascii.unhexlify(wycheproof.testcase["salt"]),
        info=binascii.unhexlify(wycheproof.testcase["info"]),
        backend=backend,
    )
    result = h.derive(binascii.unhexlify(wycheproof.testcase["ikm"]))
    assert result == binascii.unhexlify(wycheproof.testcase["okm"])



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.


import binascii

import pytest

from cryptography.hazmat.primitives import hashes, hmac

from .utils import generate_hmac_test
from ...utils import load_hash_vectors


@pytest.mark.supported(
    only_if=lambda backend: backend.hmac_supported(hashes.MD5()),
    skip_message="Does not support MD5",
)
class TestHMACMD5:
    test_hmac_md5 = generate_hmac_test(
        load_hash_vectors,
        "HMAC",
        ["rfc-2202-md5.txt"],
        hashes.MD5(),
    )


@pytest.mark.supported(
    only_if=lambda backend: backend.hmac_supported(hashes.SHA1()),
    skip_message="Does not support SHA1",
)
class TestHMACSHA1:
    test_hmac_sha1 = generate_hmac_test(
        load_hash_vectors,
        "HMAC",
        ["rfc-2202-sha1.txt"],
        hashes.SHA1(),
    )


@pytest.mark.supported(
    only_if=lambda backend: backend.hmac_supported(hashes.SHA224()),
    skip_message="Does not support SHA224",
)
class TestHMACSHA224:
    test_hmac_sha224 = generate_hmac_test(
        load_hash_vectors,
        "HMAC",
        ["rfc-4231-sha224.txt"],
        hashes.SHA224(),
    )


@pytest.mark.supported(
    only_if=lambda backend: backend.hmac_supported(hashes.SHA256()),
    skip_message="Does not support SHA256",
)
class TestHMACSHA256:
    test_hmac_sha256 = generate_hmac_test(
        load_hash_vectors,
        "HMAC",
        ["rfc-4231-sha256.txt"],
        hashes.SHA256(),
    )


@pytest.mark.supported(
    only_if=lambda backend: backend.hmac_supported(hashes.SHA384()),
    skip_message="Does not support SHA384",
)
class TestHMACSHA384:
    test_hmac_sha384 = generate_hmac_test(
        load_hash_vectors,
        "HMAC",
        ["rfc-4231-sha384.txt"],
        hashes.SHA384(),
    )


@pytest.mark.supported(
    only_if=lambda backend: backend.hmac_supported(hashes.SHA512()),
    skip_message="Does not support SHA512",
)
class TestHMACSHA512:
    test_hmac_sha512 = generate_hmac_test(
        load_hash_vectors,
        "HMAC",
        ["rfc-4231-sha512.txt"],
        hashes.SHA512(),
    )


@pytest.mark.supported(
    only_if=lambda backend: backend.hmac_supported(
        hashes.BLAKE2b(digest_size=64)
    ),
    skip_message="Does not support BLAKE2",
)
class TestHMACBLAKE2:
    def test_blake2b(self, backend):
        h = hmac.HMAC(b"0" * 64, hashes.BLAKE2b(digest_size=64), backend)
        h.update(b"test")
        digest = h.finalize()
        assert digest == binascii.unhexlify(
            b"b5319122f8a24ba134a0c9851922448104e25be5d1b91265c0c68b22722f0f29"
            b"87dba4aeaa69e6bed7edc44f48d6b1be493a3ce583f9c737c53d6bacc09e2f32"
        )

    def test_blake2s(self, backend):
        h = hmac.HMAC(b"0" * 32, hashes.BLAKE2s(digest_size=32), backend)
        h.update(b"test")
        digest = h.finalize()
        assert digest == binascii.unhexlify(
            b"51477cc5bdf1faf952cf97bb934ee936de1f4d5d7448a84eeb6f98d23b392166"
        )



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.


import binascii

import pytest

from cryptography.exceptions import InvalidTag
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives.ciphers.aead import AESCCM, AESGCM

from .utils import wycheproof_tests
from ..hazmat.primitives.test_aead import _aead_supported


@wycheproof_tests("aes_cbc_pkcs5_test.json")
def test_aes_cbc_pkcs5(backend, wycheproof):
    key = binascii.unhexlify(wycheproof.testcase["key"])
    iv = binascii.unhexlify(wycheproof.testcase["iv"])
    msg = binascii.unhexlify(wycheproof.testcase["msg"])
    ct = binascii.unhexlify(wycheproof.testcase["ct"])

    padder = padding.PKCS7(128).padder()

    cipher = Cipher(algorithms.AES(key), modes.CBC(iv), backend)
    enc = cipher.encryptor()
    computed_ct = (
        enc.update(padder.update(msg) + padder.finalize()) + enc.finalize()
    )
    dec = cipher.decryptor()
    padded_msg = dec.update(ct) + dec.finalize()
    unpadder = padding.PKCS7(128).unpadder()
    if wycheproof.valid or wycheproof.acceptable:
        assert computed_ct == ct
        computed_msg = unpadder.update(padded_msg) + unpadder.finalize()
        assert computed_msg == msg
    else:
        assert computed_ct != ct
        with pytest.raises(ValueError):
            unpadder.update(padded_msg) + unpadder.finalize()


@wycheproof_tests("aes_gcm_test.json")
def test_aes_gcm(backend, wycheproof):
    key = binascii.unhexlify(wycheproof.testcase["key"])
    iv = binascii.unhexlify(wycheproof.testcase["iv"])
    aad = binascii.unhexlify(wycheproof.testcase["aad"])
    msg = binascii.unhexlify(wycheproof.testcase["msg"])
    ct = binascii.unhexlify(wycheproof.testcase["ct"])
    tag = binascii.unhexlify(wycheproof.testcase["tag"])
    if len(iv) < 8 or len(iv) > 128:
        pytest.skip(
            "Less than 64-bit IVs (and greater than 1024-bit) are no longer "
            "supported"
        )
    if backend._fips_enabled and len(iv) != 12:
        # Red Hat disables non-96-bit IV support as part of its FIPS
        # patches.
        pytest.skip("Non-96-bit IVs unsupported in FIPS mode.")
    if wycheproof.valid or wycheproof.acceptable:
        enc = Cipher(algorithms.AES(key), modes.GCM(iv), backend).encryptor()
        enc.authenticate_additional_data(aad)
        computed_ct = enc.update(msg) + enc.finalize()
        computed_tag = enc.tag
        assert computed_ct == ct
        assert computed_tag == tag
        dec = Cipher(
            algorithms.AES(key),
            modes.GCM(iv, tag, min_tag_length=len(tag)),
            backend,
        ).decryptor()
        dec.authenticate_additional_data(aad)
        computed_msg = dec.update(ct) + dec.finalize()
        assert computed_msg == msg
    else:
        dec = Cipher(
            algorithms.AES(key),
            modes.GCM(iv, tag, min_tag_length=len(tag)),
            backend,
        ).decryptor()
        dec.authenticate_additional_data(aad)
        dec.update(ct)
        with pytest.raises(InvalidTag):
            dec.finalize()


@wycheproof_tests("aes_gcm_test.json")
def test_aes_gcm_aead_api(backend, wycheproof):
    key = binascii.unhexlify(wycheproof.testcase["key"])
    iv = binascii.unhexlify(wycheproof.testcase["iv"])
    aad = binascii.unhexlify(wycheproof.testcase["aad"])
    msg = binascii.unhexlify(wycheproof.testcase["msg"])
    ct = binascii.unhexlify(wycheproof.testcase["ct"])
    tag = binascii.unhexlify(wycheproof.testcase["tag"])
    if len(iv) < 8 or len(iv) > 128:
        pytest.skip(
            "Less than 64-bit IVs (and greater than 1024-bit) are no longer "
            "supported"
        )

    if backend._fips_enabled and len(iv) != 12:
        # Red Hat disables non-96-bit IV support as part of its FIPS
        # patches.
        pytest.skip("Non-96-bit IVs unsupported in FIPS mode.")
    aesgcm = AESGCM(key)
    if wycheproof.valid or wycheproof.acceptable:
        computed_ct = aesgcm.encrypt(iv, msg, aad)
        assert computed_ct == ct + tag
        computed_msg = aesgcm.decrypt(iv, ct + tag, aad)
        assert computed_msg == msg
    else:
        with pytest.raises(InvalidTag):
            aesgcm.decrypt(iv, ct + tag, aad)


@pytest.mark.skipif(
    not _aead_supported(AESCCM),
    reason="Requires OpenSSL with AES-CCM support",
)
@wycheproof_tests("aes_ccm_test.json")
def test_aes_ccm_aead_api(backend, wycheproof):
    key = binascii.unhexlify(wycheproof.testcase["key"])
    iv = binascii.unhexlify(wycheproof.testcase["iv"])
    aad = binascii.unhexlify(wycheproof.testcase["aad"])
    msg = binascii.unhexlify(wycheproof.testcase["msg"])
    ct = binascii.unhexlify(wycheproof.testcase["ct"])
    tag = binascii.unhexlify(wycheproof.testcase["tag"])

    if (
        wycheproof.invalid
        and wycheproof.testcase["comment"] == "Invalid tag size"
    ):
        with pytest.raises(ValueError):
            AESCCM(key, tag_length=wycheproof.testgroup["tagSize"] // 8)
        return

    aesccm = AESCCM(key, tag_length=wycheproof.testgroup["tagSize"] // 8)
    if wycheproof.valid or wycheproof.acceptable:
        computed_ct = aesccm.encrypt(iv, msg, aad)
        assert computed_ct == ct + tag
        computed_msg = aesccm.decrypt(iv, ct + tag, aad)
        assert computed_msg == msg
    elif not 7 <= len(iv) <= 13:
        with pytest.raises(ValueError):
            aesccm.decrypt(iv, ct + tag, aad)
    else:
        with pytest.raises(InvalidTag):
            aesccm.decrypt(iv, ct + tag, aad)
